#!/usr/bin/env python
#
#
#   Copyright (c) 2007 Jesse Guardiani <jesse@guardiani.us>
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation; either version 2 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
#   02111-1307, USA.
#

import math, os, pygame, gc, time, sys
from pygame.locals import *
import gtk
import globals
from db      import db      as db
from theme   import theme   as theme


def which (filename):
  if not os.environ.has_key('PATH') or os.environ['PATH'] == '':
      p = os.defpath
  else:
      p = os.environ['PATH']
  pathlist = p.split (os.pathsep)
  for path in pathlist:
      f = os.path.join(path, filename)
      if os.access(f, os.X_OK):
          return f
  return None

class Manager():
  def __init__(self):
    ''' This is the brain of the application. It handles keeping track of all the widgets,
    knowing when to draw and when not to draw a given widget, feeding the widgets with event
    data, and much more. '''
    self.screen     = None
    self.background = None
    self.db         = None
    self.view       = None
    self.dirty      = False
    self.scrobbler  = None
    self.player     = None
    self.has_a2dp        = which('a2dpd') and which('mplayer')
    self.album_view      = None
    self.nowplaying_view = None
    self.volume_view     = None
    self.screen_status   = "on" # is our display screen on, off, or dimmed?
    self.on_battery      = False # are we running on battery power or plugged in?
    self.idle_start      = 0
    self.idle_seconds    = 0

  def __load_scrobbler(self):
    if globals.ISMAEMO:
      import maemoscrobbler
      self.scrobbler = maemoscrobbler.MaemoScrobbler()

  def __load_maemo(self):
    if globals.ISMAEMO:
      import widgets
      self.clear()
      status_area = widgets.Text("CONNECTING TO DBUS")
      self.screen.blit(status_area.image,(200,200))
      pygame.display.flip()
      import maemo

  def _get_player(self):
    if globals.ISMAEMO:
      import ossoplayer
      self.player = ossoplayer.OSSOPlayer()
    else:
      import mplayer
      self.player = mplayer.Mplayer()

  #
  # Public Methods
  #
  def start(self,screen):
    import views,widgets,prefs,remote,playlist
    self.prefs    = prefs.Prefs()
    self.remote   = remote.Remote()
    self.playlist = playlist.Playlist()
    self._get_player()
    self.screen   = screen
    
    theme.set_theme(self.prefs.get('theme'))

    globals.MAINFONTCOLOR   = theme.surface.get_at((93,512))
    globals.MAINFONTBGCOLOR = theme.surface.get_at((40,582))
    
    self.background = theme.get_image('background')

    self.screen.blit(self.background,(0,0))
    pygame.display.flip()
    
    self.__load_scrobbler()
    self.__load_maemo()
    self.album_view        = views.AlbumView()
    self.nowplaying_button = widgets.NowPlayingButton()
    self.play_button       = widgets.PlayButton()
    self.a2dp_button       = widgets.A2DPButton()
    self.next_button       = widgets.NextButton()
    self.prev_button       = widgets.PrevButton()
    self.nowplaying_view   = views.NowPlayingView()
    self.volume_view       = views.VolumeView()
    self.show_albums()

  def handle_event(self,event):
    for widget in self.view.get_widgets(): widget.handle_event(event)
    self.nowplaying_button.handle_event(event)
    self.a2dp_button.handle_event(event)
    self.play_button.handle_event(event)
    self.next_button.handle_event(event)
    self.prev_button.handle_event(event)
    if event.type == KEYDOWN:
      if event.key == K_F7:    self.player.volume_increase()
      if event.key == K_F8:    self.player.volume_decrease()
      if event.key == K_LEFT:  self.playlist.prev(None,None)
      if event.key == K_RIGHT: self.playlist.next(None,None)
      if event.key == K_UP:    self.view.scroll_widget.prev()
      if event.key == K_DOWN:  self.view.scroll_widget.next()
      if event.key == K_ESCAPE:self.back()

# pause on fullscreen or headset button - headset button won't work yet (SDL's fault)
# "X11: Unknown xsym, sym = 0x1008ff6e"
      if (event.key == K_F6) or (event.key == 0x1008ff6e): self.playlist.pause()

  def back(self):
    if self.view == self.album_view: self.show_exit_confirm()
    elif self.view == self.volume_view: self.view.show_last_view()
    else: self.show_albums()

  def update(self):
    if self.is_idle():
      if not self.idle_start: self.idle_start = time.time()
      self.idle_seconds=time.time()-self.idle_start
    else: self.idle_start = 0

    for widget in self.view.get_widgets(): widget.update()
    self.nowplaying_button.update()

  def draw(self):
    for widget in self.view.get_widgets(): widget.draw(self.screen)
    self.nowplaying_button.draw(self.screen)
    self.a2dp_button.draw(self.screen)
    self.play_button.draw(self.screen)
    self.next_button.draw(self.screen)
    self.prev_button.draw(self.screen)
    self.dirty = False

  def wake_up(self):
    self.idle_start   = 0
    self.idle_seconds = 0

  def is_idle(self):
    if self.screen_status == "off": return True
    if self.dirty: return False
    for widget in self.view.get_widgets():
      if not widget.is_idle():
        return False
    return True

  def is_extended_idle(self):
    if globals.ISMAEMO:
      if manager.screen_status == "off":
        return True
    else:
      if self.idle_seconds > 5: return True
    return False

  def update_powersave(self):
    if self.player:
      if self.screen_status == "off" and self.on_battery: self.player.set_powersave(True)
      else: self.player.set_powersave(False)


  def quit(self):
    self.player.close()
    self.remote.close()
    self.playlist.save()
    self.prefs.save()

  def cleanup_current_view(self):
    if self.view and self.view != self.album_view and self.view != self.nowplaying_view and self.view != self.volume_view: self.view.close()

  def show_albums(self,selected_album=None):
    self.cleanup_current_view()
    self.view = self.album_view
    if selected_album:
      for sprite in self.album_view.album_widget.get_sprites():
        if sprite.album.name == selected_album:
          self.album_view.album_widget.set_selected_sprite(sprite)
    gc.collect()
    #print "here comes the garbage:\n\n"
    #print gc.garbage

  def show_songs(self,selected_album):
    import views
    self.cleanup_current_view()
    if not selected_album: return
    print "showing songs for album=" + selected_album.album.name
    self.view = views.SongView(selected_album.album.name)
    gc.collect()
    #print "here comes the garbage:\n\n"
    #print gc.garbage

  def show_nowplaying(self):
    if not manager.playlist.list: return
    self.cleanup_current_view()
    print "showing nowplaying"
    self.view = self.nowplaying_view
    gc.collect()

  def show_volume(self):
    if not self.volume_view: return # protect against calling before initialized
    print "showing volume"
    if self.view != self.volume_view: self.volume_view.last_view = self.view
    self.view = self.volume_view
    self.volume_view.reset_timeout()
    self.view.update()

  def show_exit_confirm(self):
    import views
    self.cleanup_current_view()
    print "showing exit confirm"
    self.view = views.ExitConfirmView()
    gc.collect()

  def clear(self):
    self.screen.blit(self.background,(0,0))

manager = Manager()
