#!/usr/bin/env python
#
#
#   Copyright (c) 2007 Jesse Guardiani <jesse@guardiani.us>
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation; either version 2 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
#   02111-1307, USA.
#

import pygame, time, gc, gobject, sys
from pygame.locals import *
import globals,widgets
from db import db as db
from manager import manager as manager
from theme   import theme   as theme


class BaseView():
  def __init__(self):
    ''' This class defines a basic interface for a view (or in web programming terms, a page).
    If Manager is the brain of the app, then each View is the brain of that screen or page. '''
    self.widgets = [] # all widgets for a view are stored within that view
    self.scroll_widget = None

  def _load_ok_button(self):
    image = theme.get_surface((75,75))
    globals.gprint("OK",image,(25,25),False,False,(255,255,255))
    button = widgets.Button(image,self._make_ok_button_callback(),True)
    button.rect.left = 45
    button.rect.top  = 200
    self.widgets.append(button)

  def _load_back_button(self):
    ''' This is the button in the lower left. It's
    a back button and clicking it goes back to
    the album screen. '''
    image = theme.get_surface((125,63))
    theme.blit(image,(35,0),'back_button')
    button = widgets.Button(image,self._make_back_button_callback())
    button.rect.left = 0 # FIXME: the controlling view should probably set the x,y pos for us
    button.rect.top  = manager.screen.get_rect().height - button.image.get_rect().height - 0
    self.widgets.append(button)

  def _make_back_button_callback(self):
    def _back_button_callback():
      return manager.back()
    return _back_button_callback

  def get_widgets(self):
    return self.widgets

  def close(self):
    self.scroll_widget = None
    for widget in self.widgets: widget.close()
    self.widgets = None


class AlbumView(BaseView):
  def __init__(self):
    ''' This is the album selection view '''
    BaseView.__init__(self)
    self.album_widget = None
    self.__load_album_widget()
    self.scroll_widget = self.album_widget
    self._load_ok_button()
    self._load_exit_button()

  def __load_album_widget(self):
    if self.album_widget:
      self.widgets.append(self.album_widget)
      return

    widget = widgets.ScrollWidget(Rect(
      130,
      manager.screen.get_rect().top,
      530,
      manager.screen.get_rect().bottom
    ))
    manager.clear()
    status_area = widgets.Text("LOADING ALBUM ART")
    manager.screen.blit(status_area.image,(200,200))
    pygame.display.flip()
    
    processtart=time.time()
    for album in self.__get_album_sprites():
      widget.add(album)
    print "load albums took:" + str(time.time()-processtart)
    self.album_widget = widget
    self.widgets.append(widget)

  def __get_album_sprites(self):
    processtart=time.time()
    list = db.get_albums()
    print "album query took:" + str(time.time()-processtart)
    processtart=time.time()
    last_time = processtart
    progress_bar = widgets.PercentBar(0)
    progress_bar.set_selected(True)
    c = 0
    list2 = []
    for row in list:
      c = c + 1
      list2.append(widgets.AlbumArtist(row['album'],row['artist']))
      current_time = time.time()
      if current_time - last_time >= 1: # update only once per second
        progress_bar.set_percent(c * 100 / len(list))
        manager.screen.blit(progress_bar.image,(200,300))
        pygame.display.flip()
        last_time = current_time
    print "album search took:" + str(time.time()-processtart)
    return list2

  def _make_ok_button_callback(self):
    def _ok_button_callback():
      sel_sprite = self.scroll_widget.get_selected_sprite()
      if not sel_sprite:
        self.scroll_widget.stop_animation()
        return None
      else:
        return manager.show_songs(sel_sprite)
    return _ok_button_callback

  def _load_exit_button(self):
    ''' This is the button in the lower left. It's
    an exit button. '''
    image = theme.get_surface((125,63))
    theme.blit(image,(35,0),'exit_button')
    button = widgets.Button(image,self._make_exit_button_callback())
    button.rect.left = 0 # FIXME: the controlling view should probably set the x,y pos for us
    button.rect.top  = manager.screen.get_rect().height - button.image.get_rect().height - 0
    self.widgets.append(button)

  def _make_exit_button_callback(self):
    def _exit_button_callback():
      return manager.back()
    return _exit_button_callback


class SongView(BaseView):
  def __init__(self,album):
    ''' This is the song selection view '''
    BaseView.__init__(self)
    self.album       = album
    self.__load_song_widget()
    self._load_ok_button()
    self._load_back_button()
    self._load_songalbum_button()

  def __load_song_widget(self):
    if self.scroll_widget != None:
      self.widgets.append(self.scroll_widget)
      return

    widget = widgets.ScrollWidget(Rect(
      130,
      manager.screen.get_rect().top,
      530,
      manager.screen.get_rect().bottom
    ))
    manager.clear()
    status_area = widgets.Text("LOADING SONGS")
    manager.screen.blit(status_area.image,(200,200))
    pygame.display.flip()
    
    processtart=time.time()
    for song in self.__get_song_sprites():
      widget.add(song)
    print "load songs took:" + str(time.time()-processtart)
    self.scroll_widget = widget
    self.widgets.append(widget)
    sprites = self.scroll_widget.get_sprites()
    self.scroll_widget.set_selected_sprite(sprites[0])

  def __get_song_sprites(self):
    last_time = time.time()
    progress_bar = widgets.PercentBar(0)
    progress_bar.set_selected(True)
    c = 0
    l = []
    list = db.get_songs(self.album)
    for song in list:
      c = c + 1
      l.append(widgets.Song(song['track'],song['title'],song['path']))
      current_time = time.time()
      if current_time - last_time >= 1: # update only once per second
        progress_bar.set_percent(c * 100 / len(list))
        manager.screen.blit(progress_bar.image,(200,300))
        pygame.display.flip()
        last_time = current_time
    return l

  def _play_continuous(self):
    if not self.scroll_widget.get_selected_sprite(): return
    manager.playlist.clear()
    for sprite in self.scroll_widget.get_sprites():
      manager.playlist.add(sprite.path)
    manager.playlist.play(self.scroll_widget.get_selected_sprite().path)
    manager.show_nowplaying()

  def _make_ok_button_callback(self):
    def _ok_button_callback():
      if not self.scroll_widget.get_selected_sprite():
        self.scroll_widget.stop_animation()
      else:
        self._play_continuous()
    return _ok_button_callback

  def _load_songalbum_button(self):
    ''' This is the album picture in the upper left.
    Clicking it should (eventually) bring up a hi res version of
    the album art. '''
    image = theme.get_surface((125,160+20))
    print "songalbum: " + self.album
    for sprite in manager.album_view.album_widget.sprites():
      if sprite.album.name == self.album:
        image.blit(sprite.album.image,(0,0))

    artist = db.artist_of_album(self.album)
    globals.gprint(artist,image,(2,162),16,False,(255,255,255))
    button = widgets.Button(image,self._make_songalbum_button_callback())
    button.rect.left = 0
    button.rect.top  = 0
    self.widgets.append(button)

  def _make_songalbum_button_callback(self):
    def _songalbum_button_callback():
      print "show hires album"
      #return manager.show_songs(self.album_widget.get_selected_sprite())
    return _songalbum_button_callback


class NowPlayingView(BaseView):
  def __init__(self):
    ''' This is the nowplaying selection view '''
    BaseView.__init__(self)
    self.album = None
    self.dirty = False
    self._load_scroll_widget()
    self._load_ok_button()
    self._load_back_button()
    self.update()

  def _load_scroll_widget(self):
    if self.scroll_widget != None:
      self.widgets.remove(self.scroll_widget)
      self.scroll_widget = None
    gc.collect()

    widget = widgets.ScrollWidget(Rect(
      130,
      manager.screen.get_rect().top,
      530,
      manager.screen.get_rect().bottom
    ))
    
    manager.clear()
    status_area = widgets.Text("LOADING NOW PLAYING")
    manager.screen.blit(status_area.image,(200,200))
    pygame.display.flip()

    processtart=time.time()
    for song in self.__get_song_sprites():
      widget.add(song)
    print "load songs took:" + str(time.time()-processtart)
    self.scroll_widget = widget
    self.widgets.append(widget)
    self.dirty = False

  def __get_song_sprites(self):
    last_time = time.time()
    progress_bar = widgets.PercentBar(0)
    progress_bar.set_selected(True)
    c = 0
    l = []
    list = manager.playlist.list
    for path in list:
      c = c + 1
      song = db.song_of_path(path)[0]
      l.append(widgets.Song(song['track'],song['title'],song['path']))
      current_time = time.time()
      if current_time - last_time >= 1: # update only once per second
        progress_bar.set_percent(c * 100 / len(list))
        manager.screen.blit(progress_bar.image,(200,300))
        pygame.display.flip()
        last_time = current_time
    return l

  def _play_continuous(self):
    if not self.scroll_widget.get_selected_sprite(): return
    manager.playlist.play(self.scroll_widget.get_selected_sprite().path)

  def _make_ok_button_callback(self):
    def _ok_button_callback():
      if not self.scroll_widget.get_selected_sprite():
        self.scroll_widget.stop_animation()
      else:
        self._play_continuous()
    return _ok_button_callback

  def update(self):
    print "now update: %s" % (manager.playlist.current,)
    if self.dirty: self._load_scroll_widget()
    manager.nowplaying_button.update()
    sprites = self.scroll_widget.get_sprites()
    for sprite in sprites:
      if sprite.path == manager.playlist.list[manager.playlist.current]:
        self.scroll_widget.set_selected_sprite(sprite)
        sprite.set_selected(True)
        print "selectednow: " + sprite.path
      else:
        sprite.set_selected(False)


class VolumeView(BaseView):
  def __init__(self):
    ''' This is the nowplaying selection view '''
    BaseView.__init__(self)
    self.album         = None
    self.last_view     = None
    self.timeout       = 3000 # jump to last_view after timeout milliseconds
    self.timeout_cb    = None
    self.cur_percent   = manager.player.volume
    self._load_scroll_widget()
    self._load_ok_button()
    self._load_back_button()
    self.update()

  def _load_scroll_widget(self):
    widget = widgets.ScrollWidget(Rect(
      130,
      manager.screen.get_rect().top,
      530,
      manager.screen.get_rect().bottom
    ))
    
    processtart=time.time()
    for sprite in self.__get_volume_sprites():
      widget.add(sprite)
    print "load volume took:" + str(time.time()-processtart)
    self.scroll_widget = widget
    self.widgets.append(widget)

  def __get_volume_sprites(self):
    l = []
    for c in range(0,101,10):
      l.append(widgets.PercentBar(c))
    return l

  def _make_ok_button_callback(self):
    def _ok_button_callback():
      self.reset_timeout()
      sel_sprite = self.scroll_widget.get_selected_sprite()
      if not sel_sprite:
        self.scroll_widget.stop_animation()
      else:
        if self.cur_percent == sel_sprite.percent:
          self.show_last_view()
        else:
          manager.player.set_volume(sel_sprite.percent)
          self.update()
    return _ok_button_callback

  def reset_timeout(self):
    if self.timeout_cb: gobject.source_remove(self.timeout_cb)
    self.timeout_cb = gobject.timeout_add(self.timeout, self.show_last_view)

  def show_last_view(self):
    if self.last_view and manager.view == manager.volume_view:
      if not self.scroll_widget.is_idle(): return self.reset_timeout()
      manager.view = self.last_view
      self.last_view = None
    gobject.source_remove(self.timeout_cb)
    self.timeout_cb = None
    manager.dirty = True

  def update(self):
    print "volume update:"
    self.cur_percent = manager.player.volume
    sprites = self.scroll_widget.get_sprites()
    for sprite in sprites:
      if sprite.percent == manager.player.volume:
        self.scroll_widget.set_selected_sprite(sprite)
        sprite.set_selected(True)
        print "selectednow: %s" % (sprite.percent,)
      else:
        sprite.set_selected(False)


class ExitConfirmView(BaseView):
  def __init__(self):
    ''' This is the exit confirmation dialog/view '''
    BaseView.__init__(self)
    self.album         = None
    self._load_ok_button()
    self._load_cancel_button()
    status_area = widgets.Text("      Quit Kagu?         ")
    status_area.rect.left = 200
    status_area.rect.top  = 200
    self.widgets.append(status_area)

  def _make_ok_button_callback(self):
    def _ok_button_callback():
      manager.quit()
      sys.exit(0)
    return _ok_button_callback

  def _make_cancel_button_callback(self):
    def _cancel_button_callback():
      manager.show_albums()
    return _cancel_button_callback

  def _load_ok_button(self):
    image = theme.get_surface((75,75))
    globals.gprint("OK",image,(25,25),False,False,(255,255,255))
    button = widgets.Button(image,self._make_ok_button_callback(),True)
    button.rect.left = 200
    button.rect.top  = 300
    self.widgets.append(button)

  def _load_cancel_button(self):
    image = theme.get_surface((200,75))
    globals.gprint("CANCEL",image,(25,25),False,False,(255,255,255))
    button = widgets.Button(image,self._make_cancel_button_callback(),False)
    button.rect.left = 400
    button.rect.top  = 300
    self.widgets.append(button)


